/*****************************************************
**********     Charles Bellemare      ****************
**********     Luc Bissonnette        ****************
**********     Sabine Kroger          ****************
**********     Universite Laval       ****************
**********     Dept. of Economics     ****************
**********     February 13th 2019     ****************
*****************************************************/                             

* Description: 
* This fonction can be used to simulate statistical power for a class of 
* popular data-generating processes encountered in experimental economics 
* using a single command line. It requires the user to specify details concerning the
* experimental designs such as the number of subjects, 
* number of periods, WS or BS design, balance of WS design and so on.
*
* Dependency:
* This program calls the command "simulate", in turn calling the program "powerBBK_onerep »
* "powerBBK_onerep » is defined in "powerBBK_onerep.ado". 
*
* Note that this two .ado are necessary, as Stata command "simulate" cannot use locally defined programs.
*
* Bug fix
* 1.03  The critical level requested was not properly passed to the "one rep" function.
*



capture program drop powerBBK

program define powerBBK
  version 12
  syntax newvarlist(max = 1),  BUDGET(numlist >=0)   T(integer) /// N(integer) ///
     BETA(numlist) DESIGN(string) MUVAR(real) EPSVAR(numlist) COMMand(string) ///
   [ALLOCation(numlist >=0) COSTCONTrol(real 1) COSTTREATment(real 1)  ///
    REPlications(integer 1000) SEED(integer 123456)  ///
    PANEL CRITICAL(real 0.05)   ORDEReffect(real -999) /// 
    DISTRIBution(string) CONTinuous(string)] 
   

   tokenize `varlist' 
   local newname = "`1'"

   //To test only once for panel
   local ispanel = "`panel'" == "panel"
   
   
   
   //Catching some errors
   if(wordcount("`epsvar'") > 2){
      di as error "Too many values (>2) used for the variances of the error terms."  
      error 498
   }   


   if("`command'"=="probit" & (wordcount("`epsvar'") == 2)  ){
	 di as error "Warning: heteroskedasticity not allowed with option probit"
	 error 498  
   } 
   
   if("`command'"=="logit" & (wordcount("`epsvar'") == 2)  ){
	 di as error "Warning: heteroskedasticity not allowed with option logit"
	 error 498  
   } 

   //Order effects
   local isorder = `ordereffect' != -999
   if(`isorder'){
	  //Order effects for within only. 
      if("`design'" != "within"){
       di as error "Order effects for within design only"
	   error 498   
      }
	  //No order effects for rank. 
      if("`command'" == "rank"){
       di as error "Order effect not available for rank tests"
	   error 498   
      }
   
      if(wordcount("`beta'") > 2)  {
	   di as error "Order effect not available for time-varying treatment effects."
	   error 498  
     } 
   
      //Order effect of 1 does not make sense.
      if(`ordereffect' == 1){
       di as error "Order effect of 1 specified, implying no order effect."
      }
   }
   	

   
   //Both or only within ?
   quietly{
   if("`design'" == "within"){
    local designlist = "within" 
   }
   else if("`design'" == "between"){
   local designlist = "between"
   }
   else if("`design'" == "both"){
   local designlist = "within between"
   }
   else {
   no di as error "Invalid design type, must be either: within, between, both"  
   error 498
   }
  }
   

   //Default allocation is 0.5
  if("`allocation'" == ""){
   numlist "0.5"
   local allocation = r(numlist)
   }  
  
   // Testing the number of parameters
  if("`design'" == "within"){   
    if(wordcount("`beta'") != 2){
       if(wordcount("`allocation'") > 1  ){
        di as error "Multiple allocations and time-varying treatment not allowed with within design."  
		error 498
	   }
	   if(wordcount("`beta'") != round(`t'*`=1-`allocation'')+1){
         di as error "Incorrect number of values for beta parameters (must be either 2 or 'treated rounds + 1' ) in a within design."  
         error 498
     }
   }
  }
   else if("`design'" == "between"){
      if(wordcount("`beta'") != 2){
       if(wordcount("`beta'") != `t' + 1){
         di as error "Incorrect number of values for beta parameters (must be either 2 or t+1) in a between design."  
         error 498
     }
   }  
   }
    else if("`design'" == "both"){
	 if(wordcount("`beta'") != 2){
         di as error "Time-varying treatment not allowed with option both. "  
         error 498
     }
	}
   
   
   
   if(!`ispanel'  & (`muvar'>0)){
     di " "
	 di as error "Warning: individual heterogeneity generated, but not taken into account in estimation"  
   } 
  

   //Description
   di " "
   di as input "Version 1.03"
   di as input "February 13th 2019"
   di as input "Please cite as Bellemare, Charles, Luc Bissonnette and Sabine Kroger. 2016. Statistical power of within and between-subjects in economic experiments"
   di " "
   di as input "Summary of the simulation "  
   di as text "{hline 42}"
     
   //Type of analysis
   if("`command'" == "regress"){
    if(`ispanel'){
      if(wordcount("`epsvar'") ==2)  di as input "Mixed-effects linear-regression"
      else                           di as input "Random-effects GLS regression"
    }
    else {
      if(wordcount("`epsvar'") ==2) di as input "GLS regression" 
      else                          di as input "Linear regression with clusters"
    }
   }
  else if("`command'" == "probit"){
   di as input  "Probit analysis"
   }  
  else if("`command'" == "logit"){
   di as input  "Logit analysis"
   }  
  else if("`command'" == "tobit"){
   di as input  "Tobit analysis"
   } 
  else if("`command'" == "rank"){
   di as input "Rank test : Mann-Whitney/Wilcoxon T-test"
   }        
  else {
   di as error "Invalid command used, see help for list of valid commands"  
   error 498
  }
  if(`isorder'){
     di as input  "Testing for an order effect"
  }
   
   

 


  
  
      
  di as text "{hline 42}"
   di as input "Design parameters"
   
   if("`design'"=="both"){
     di as text "Designs simulated: " as result _column(36) "both"
   }
   else{
     di as text "Design simulated: " as result _column(36) "`design' "
   }
   
   di as text "Number of rounds: " as result _column(36) "`t'"
   di as text "Budget list " as result _column(36) "`budget'"
   di as text "Fractions allocated to control " as result _column(36) "`allocation' "
   di as text "Cost of control: " as result _column(36)   "`costcontrol' "
   di as text "Cost of treatment: " as result _column(36) "`costtreatment' "

   
   if("`continuous'" == ""){
    di as text "Continuous treatment: " as result _column(36) "no"
   }
   else{
    di as text "Continuous treatment: "  as result _column(36) "`continuous' "
   }
   //di as text "Number of individuals: " as result _column(36) "`n' "

   
 //  if(`control' <= 1) di as text "Fraction of rounds/ind. in control" as result _column(36) "`control'" 
 //  else di as text "Number of round(s)/ind.  in control" as result _column(36) "`control'" 
 
 
 
   di " "
   di as input "Simulation parameters"
   
   di as text "Value constant term: " as result _column(36) word("`beta'",1)
   if(wordcount("`beta'") == 2){
     di as text "Treatment effect (all rounds): " as result _column(36) word("`beta'",2)
   }
   else{
    forvalues i = 2/`=wordcount("`beta'")'{
      di as text "Treatment effect (round=`=`i'-1'): " as result _column(36) word("`beta'",`i') 
    }
   }
   
   if(`isorder'){
     di as text "Order effect (if last): " as result _column(36) "`ordereffect'" 
   }
   
   
   di as text "Variance of individual effects: " as result _column(35) "`muvar'"
   di as text "Variance of noise: " as result _column(35) "`epsvar' "
   di " "
   di as input "Econometric specification"
   if(`ispanel') di as text "Individual heterogeneity:" as result _column(36) "yes"
   else        di as text "Individual heterogeneity:" as result _column(36) "no"
   
   if(wordcount("`epsvar'") ==2) di as text "Treatment heterosckedasticity:" as result _column(36) "yes"
   else        di as text "Treatment heterockedasticity:" as result _column(36) "no"

   
   di as text "Power for a critical value of: " as result _column(35) "`critical'"
di as text "{hline 42}"    
      
    
      
foreach thisdesign in `designlist'{   
 di " "
 no di as input "Simulation of a `thisdesign' design" 
 preserve
 matrix results_`thisdesign' = J(1,9,0)
   

   
 //Below this, we must now split the sample size accordingly...
 local firstline = 1
 foreach bud in `budget'{
   foreach alloc in `allocation'{

   
if("`continuous'" != ""){   

  local numtreatment = int(`bud'/(`costtreatment'*`t'))


   local numcontrol   = 0
   local sampsize     = `numtreatment'
   local cost         = `numtreatment' * `costtreatment' * `t'


   
   local itreated       = `numtreatment'
   local inottreated    = `numcontrol'
   
  local roundcontrol     = 0
  local roundtreated     = `t'
   
}   
else{     
 if("`thisdesign'" == "between"){
 //Between case 


  //local numcontrol =  int(`alloc'*`bud'/(`costcontrol'*`t')) 
  //local numtreatment = int((`bud' - `numcontrol'* `costcontrol'*`t')/(`costtreatment'*`t'))
  
  local numcontrol   = round(`bud'/( `t'*((1-`alloc')/`alloc'* `costtreatment' +  `costcontrol')  ))
  local numtreatment = int((`bud' - `numcontrol'* `costcontrol'*`t')/(`costtreatment'*`t'))
     
  local sampsize     = `numcontrol' + `numtreatment'
  local cost      = (`numcontrol'*`costcontrol' + `numtreatment'*`costtreatment')*`t'

 
  local itreated          = `numtreatment'
  local inottreated = `numcontrol'
  local roundcontrol     = `t'
  local roundtreated   = `t'

   
  
  }
 else{
	local numcontrol = round(`alloc'*`t')
	local numtreatment =  (`t'-`numcontrol')
	local indcost = `numcontrol'* `costcontrol' + `numtreatment'*`costtreatment'
	local sampsize = int(`bud'/`indcost')
	local cost = `indcost'*`sampsize'
   
   
    local inottreated   = `sampsize'
    local itreated      = `sampsize'
    local roundcontrol  = `numcontrol'
    local roundtreated  = `numtreatment'
   
   }   
 }  

   
   clear
   if(wordcount("`beta'") == 2){
   qui simulate power_lt=r(plt) power=r(power)  power_ut=r(put), reps(`replications'): ///
   powerBBK_onerep, n(`sampsize') t(`t') design(`thisdesign') beta(`beta') muvar(`muvar') epsvar(`epsvar') ///
   control(`numcontrol') comm(`command') panel(`ispanel') continuous(`continuous') ordereffect(`ordereffect') critical(`critical')
   }
   else{
   qui simulate power=r(power), reps(`replications'): ///
   powerBBK_onerep, n(`sampsize') t(`t') design(`thisdesign') beta(`beta') muvar(`muvar') epsvar(`epsvar') ///
   control(`numcontrol') comm(`command') panel(`ispanel') continuous(`continuous') ordereffect(`ordereffect') critical(`critical')
   }


   
   gen budget = `bud'
   gen N = `sampsize'
      
   gen n_ctrl  = `inottreated'
   gen n_treat      = `itreated'
   gen t_ctrl = `roundcontrol'
   gen t_treat  = `roundtreated'
    
   //gen control = `numcontrol'
   //gen treatment = `numtreatment'
   gen cost = `cost'
   
   if(!`isorder'){
    if(real(word("`beta'", 2)) > 0 ){ 
	 capture gen power_lt = .
     rename power_lt s_error
	} 
    else {
     capture gen power_ut = .
     rename power_ut s_error
	}
   }
   else {
    if(`ordereffect' > 1 ){ 
	 capture gen power_lt = .
     rename power_lt s_error
	} 
	else if(`ordereffect' < 1 ) {
     capture gen power_ut = .
     rename power_ut s_error
	}
	else {
    capture gen s_error	= power 
   }
   
   }
   
   
   collapse *
   
   mkmat budget N n_* t_* cost power s_error, matrix(A)
   matrix results_`thisdesign' = results_`thisdesign' \ A
    

  
   if(`firstline'){
   di "Simulated command: $simcmd"
   di " "
   //if("`thisdesign'" == "within"){
   // di "Columns control and treatment refer to the" as result " number of rounds"
  // }
   //else{
   // di "Columns control and treatment refer to the" as result " number of participants"
  // }
   else{
    di ""
   }
   
   
   matlist A , name(column) //title(Design: `thisdesign')
   local firstline = 0
   }
   else{
    matlist A, name(none) noblank
   } 
   
  }
  
 }  
 

    clear
   restore
   qui svmat results_`thisdesign'
   
   capture rename results_`thisdesign'1 budget_`thisdesign'
   rename results_`thisdesign'2 N_`thisdesign'
   rename results_`thisdesign'3 `newname'_n_ctrl_`thisdesign'
   rename results_`thisdesign'4 `newname'_n_treat_`thisdesign'
   rename results_`thisdesign'5 `newname'_t_ctrl_`thisdesign'  
   rename results_`thisdesign'6 `newname'_t_treat_`thisdesign'
   
   rename results_`thisdesign'7 `newname'_cost_`thisdesign'
   rename results_`thisdesign'8 `newname'_power_`thisdesign'
   rename results_`thisdesign'9 `newname'_signerror_`thisdesign'
   
  

}
 if("`design'" == "both") drop budget_between
capture rename budget_ budget
if(_rc){
 drop budget_
 
 }

qui drop if _n == 1
 
  
end program

